@@ -1,6 +1,6 @@
 /* tifffastcrop
 
- v. 1.3.7
+ v. 1.3.8
 
  Copyright (c) 2013-2016 Christophe Deroulers
 
@@ -510,7 +510,7 @@ static int cpTiles2Strip(TIFF* in, uint32 xmin, uint32 ymin,
 	tsize_t intilewidthinbytes = TIFFTileRowSize(in);
 	uint32 y;
 	unsigned char * inbuf, * bufp= outbuf;
-	int success = 1;
+	int error = 0;
 
 	TIFFGetField(in, TIFFTAG_TILEWIDTH, &intilewidth);
 	TIFFGetField(in, TIFFTAG_TILELENGTH, &intilelength);
@@ -521,7 +521,7 @@ static int cpTiles2Strip(TIFF* in, uint32 xmin, uint32 ymin,
 	if (!inbuf) {
 		TIFFError(TIFFFileName(in),
 				"Error, can't allocate space for image buffer");
-		return (0);
+		return (EXIT_INSUFFICIENT_MEMORY);
 	}
 
 	for (y = ymin ; y < ymin + length + intilelength ; y += intilelength) {
@@ -560,7 +560,7 @@ static int cpTiles2Strip(TIFF* in, uint32 xmin, uint32 ymin,
 				    "Error, can't read tile at "
 				    UINT32_FORMAT ", " UINT32_FORMAT,
 				    xminoftile, yminoftile);
-				success = 0;
+				error = EXIT_IO_ERROR;
 				goto done;
 			}
 
@@ -576,7 +576,7 @@ static int cpTiles2Strip(TIFF* in, uint32 xmin, uint32 ymin,
 
 	done:
 	_TIFFfree(inbuf);
-	return success;
+	return error;
 }
 
 static int cpStrips2Strip(TIFF* in,
@@ -590,7 +590,7 @@ static int cpStrips2Strip(TIFF* in,
 	tsize_t inwidthinbytes = TIFFScanlineSize(in);
 	uint32 y;
 	unsigned char * inbuf, * bufp= outbuf;
-	int success = 1;
+	int error = 0;
 
 	TIFFGetFieldDefaulted(in, TIFFTAG_COMPRESSION, &input_compression);
 	inbufsize= TIFFScanlineSize(in);  /* not malloc because
@@ -599,7 +599,7 @@ static int cpStrips2Strip(TIFF* in,
 	if (!inbuf) {
 		TIFFError(TIFFFileName(in),
 				"Error, can't allocate space for image buffer");
-		return (0);
+		return (EXIT_INSUFFICIENT_MEMORY);
 	}
 
 	/* When compression method doesn't support random access: */
@@ -616,7 +616,7 @@ static int cpStrips2Strip(TIFF* in,
 					    "Error, can't read scanline at "
 					    UINT32_FORMAT " for exhausting",
 					    y);
-					success = 0;
+					error = EXIT_IO_ERROR;
 					goto done;
 				} else
 					*y_of_last_read_scanline= y;
@@ -629,7 +629,7 @@ static int cpStrips2Strip(TIFF* in,
 				TIFFError(TIFFFileName(in),
 				    "Error, can't read scanline at "
 				    UINT32_FORMAT " for exhausting", y);
-				success = 0;
+				error = EXIT_IO_ERROR;
 				goto done;
 			} else
 				*y_of_last_read_scanline= y;
@@ -642,7 +642,7 @@ static int cpStrips2Strip(TIFF* in,
 			TIFFError(TIFFFileName(in),
 			    "Error, can't read scanline at "
 			    UINT32_FORMAT " for copying", y);
-			success = 0;
+			error = EXIT_IO_ERROR;
 			goto done;
 		} else
 			*y_of_last_read_scanline= y;
@@ -655,7 +655,7 @@ static int cpStrips2Strip(TIFF* in,
 
 	done:
 	_TIFFfree(inbuf);
-	return success;
+	return error;
 }
 
 
@@ -764,7 +764,7 @@ static int makeExtractFromTIFFFile(const char * infilename,
 			fprintf(stderr, "Internal error.\n");
 			return EXIT_UNHANDLED_FILE_TYPE;
 		}
-		if (! success) {
+		if (!success) {
 			TIFFClose(in);
 			return EXIT_UNHANDLED_FILE_TYPE;
 		}
@@ -892,6 +892,7 @@ static int makeExtractFromTIFFFile(const char * infilename,
 		uint16 bytesperpixel = (bitspersample/8) * spp;
 		tsize_t outscanlinesizeinbytes =
 		    requestedwidth * bytesperpixel;
+		int error = 0;
 
 		cinfo.err = jpeg_std_error(&jerr);
 		jpeg_create_compress(&cinfo);
@@ -908,19 +909,18 @@ static int makeExtractFromTIFFFile(const char * infilename,
 		    TRUE /* limit to baseline-JPEG values */);
 		jpeg_start_compress(&cinfo, TRUE);
 
-
 		if (((TIFFIsTiled(in) &&
-		      cpTiles2Strip(in,
+		      !(error = cpTiles2Strip(in,
 			    requestedxmin, requestedymin,
 			    requestedwidth, requestedlength,
-			    outbuf, outscanlinesizeinbytes, bytesperpixel)) ||
+			    outbuf, outscanlinesizeinbytes, bytesperpixel))) ||
 		     (!TIFFIsTiled(in) &&
-		      cpStrips2Strip(in,
+		      !(error = cpStrips2Strip(in,
 			    requestedxmin, requestedymin,
 			    requestedwidth, requestedlength,
 			    outbuf, outscanlinesizeinbytes, bytesperpixel,
 			    &y_of_last_read_scanline,
-			    inimagelength)))) {
+			    inimagelength))))) {
 			if (verbose)
 				fprintf(stderr, "Extract prepared.\n");
 
@@ -945,13 +945,11 @@ static int makeExtractFromTIFFFile(const char * infilename,
 			free(row_pointers);
 		} else {
 			fprintf(stderr, "Error, can't write extract.\n");
-			return_code = EXIT_IO_ERROR;
+			return_code = error;
 		}
 
 		jpeg_finish_compress(&cinfo);
 		fclose(out);
-		if (return_code == 0 && verbose)
-			fprintf(stderr, "Extract written.\n");
 		jpeg_destroy_compress(&cinfo);
 		}
 		break;
@@ -962,6 +960,7 @@ static int makeExtractFromTIFFFile(const char * infilename,
 		uint16 bitsperpixel = bitspersample * spp;
 		tsize_t outscanlinesizeinbytes = computeWidthInBytes(
 		    requestedwidth, bitsperpixel);
+		int error = 0;
 
 		png_structp png_ptr = png_create_write_struct(
 		    PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
@@ -1015,22 +1014,20 @@ static int makeExtractFromTIFFFile(const char * infilename,
 		/*png_set_packing(png_ptr);*/ /* Use *only* if bits are 
 		 not yet packed */
 
-fprintf(stderr, "outscanlinesizeinbytes=%zu\n", outscanlinesizeinbytes);
-
 		if (((TIFFIsTiled(in) &&
-		      cpTiles2Strip(in,
+		      !(error = cpTiles2Strip(in,
 			    requestedxmin, requestedymin,
 			    requestedwidth, requestedlength,
 			    outbuf, outscanlinesizeinbytes,
-			    bitsperpixel)) ||
+			    bitsperpixel))) ||
 		     (!TIFFIsTiled(in) &&
-		      cpStrips2Strip(in,
+		      !(error = cpStrips2Strip(in,
 			    requestedxmin, requestedymin,
 			    requestedwidth, requestedlength,
 			    outbuf, outscanlinesizeinbytes,
 			    bitsperpixel,
 			    &y_of_last_read_scanline,
-			    inimagelength)))) {
+			    inimagelength))))) {
 			if (verbose)
 				fprintf(stderr, "Extract prepared.\n");
 
@@ -1040,12 +1037,10 @@ fprintf(stderr, "outscanlinesizeinbytes=%zu\n", outscanlinesizeinbytes);
 			    y++, row_pointer += outscanlinesizeinbytes)
 				png_write_row(png_ptr, row_pointer);
 		} else
-			return_code = EXIT_IO_ERROR;
+			return_code = error;
 
 		png_write_end(png_ptr, info_ptr);
 		fclose(out);
-		if (return_code == 0 && verbose)
-			fprintf(stderr, "Extract written.\n");
 		png_destroy_write_struct(&png_ptr, (png_infopp) & info_ptr);
 		png_destroy_info_struct(png_ptr, (png_infopp) & info_ptr);
 		}
@@ -1056,6 +1051,7 @@ fprintf(stderr, "outscanlinesizeinbytes=%zu\n", outscanlinesizeinbytes);
 		{
 		uint16 bitsperpixel = bitspersample * spp;
 		tsize_t outscanlinesizeinbytes;
+		int error = 0;
 
 		tiffCopyFieldsButDimensions(in, out);
 		TIFFSetField(out, TIFFTAG_IMAGEWIDTH, requestedwidth);
@@ -1067,18 +1063,16 @@ fprintf(stderr, "outscanlinesizeinbytes=%zu\n", outscanlinesizeinbytes);
 			 * otherwise, ScanlineSize may be wrong */
 		outscanlinesizeinbytes = TIFFScanlineSize(out);
 
-fprintf(stderr, "outscanlinesizeinbytes=%zu\n", outscanlinesizeinbytes);
-
 		if (((TIFFIsTiled(in) &&
-		      cpTiles2Strip(in, requestedxmin,
+		      !(error = cpTiles2Strip(in, requestedxmin,
 			requestedymin, requestedwidth, requestedlength,
-			outbuf, outscanlinesizeinbytes, bitsperpixel)) ||
-		     (! TIFFIsTiled(in) &&
-		      cpStrips2Strip(in, requestedxmin,
+			outbuf, outscanlinesizeinbytes, bitsperpixel))) ||
+		     (!TIFFIsTiled(in) &&
+		      !(error = cpStrips2Strip(in, requestedxmin,
 			requestedymin, requestedwidth, requestedlength,
 			outbuf, outscanlinesizeinbytes, bitsperpixel,
 			&y_of_last_read_scanline,
-			inimagelength)))) {
+			inimagelength))))) {
 			if (verbose)
 				fprintf(stderr, "Extract prepared.\n");
 
@@ -1091,7 +1085,8 @@ fprintf(stderr, "outscanlinesizeinbytes=%zu\n", outscanlinesizeinbytes);
 				fprintf(stderr, "Extract written to "
 					"output file \"%s\".\n",
 					TIFFFileName(out));
-		}
+		} else
+			return_code = error;
 
 		TIFFClose(out);
 		}
@@ -1104,6 +1099,8 @@ fprintf(stderr, "outscanlinesizeinbytes=%zu\n", outscanlinesizeinbytes);
 		return EXIT_UNHANDLED_FILE_TYPE;
 	}
 
+	if (return_code == 0 && verbose)
+		fprintf(stderr, "Extract written.\n");
 	free(outbuf);
 	TIFFClose(in);
 	return return_code;
@@ -1216,7 +1213,7 @@ static int processCompressOptions(char* opt)
 			cp = strchr(cp+1,':');
 		}
 	} else if (strncasecmp(opt, "g3", 2) == 0) {
-		if (! processG3Options(opt))
+		if (!processG3Options(opt))
 			return 0;
 		defcompression = COMPRESSION_CCITTFAX3;
 	} else if (strcasecmp(opt, "g4") == 0) {
@@ -1227,11 +1224,11 @@ static int processCompressOptions(char* opt)
 			defpredictor = atoi(cp+1);
 		defcompression = COMPRESSION_LZW;
 	} else if (strncasecmp(opt, "zip", 3) == 0) {
-		if (! processZIPOptions(opt))
+		if (!processZIPOptions(opt))
 			return 0;
 		defcompression = COMPRESSION_ADOBE_DEFLATE;
 	}/* else if (strncasecmp(opt, "lzma", 4) == 0) {
-		if (! processZIPOptions(opt) )
+		if (!processZIPOptions(opt) )
 			return 0;
 		defcompression = COMPRESSION_LZMA;
 	}*/ else if (strncasecmp(opt, "jbig", 4) == 0) {
@@ -1351,7 +1348,7 @@ int main(int argc, char * argv[])
 	arg++;
 	}
 
-	if (argc > 1 && ! seen_extract_geometry_on_the_command_line) {
+	if (argc > 1 && !seen_extract_geometry_on_the_command_line) {
 		fprintf(stderr, "The extract's position and size must be specified on the command line as argument to the '-E' option. Aborting.\n");
 		return EXIT_GEOMETRY_ERROR;
 	}
