@@ -1,6 +1,6 @@
 /* tiffmakemosaic
 
- v. 1.3.7
+ v. 1.3.8
 
  Copyright (c) 2012-2016 Christophe Deroulers
 
@@ -353,7 +353,7 @@ cpTiles2Strip(TIFF* in, void * ambiguous_out,
 	tsize_t outscanlinesizeinbytes;
 	uint32 y;
 	unsigned char * inbuf, * bufp= outbuf;
-	int success = 1;
+	int error = 0;
 
 	if (output_to_jpeg_rather_than_tiff) {
 		p_cinfo = (struct jpeg_compress_struct *) ambiguous_out;
@@ -366,7 +366,7 @@ cpTiles2Strip(TIFF* in, void * ambiguous_out,
 	TIFFGetField(in, TIFFTAG_TILEWIDTH, &intilewidth);
 	TIFFGetField(in, TIFFTAG_TILELENGTH, &intilelength);
 
-	if (! testAndFixParameters(in, output_to_jpeg_rather_than_tiff,
+	if (!testAndFixParameters(in, output_to_jpeg_rather_than_tiff,
 		    &outscanlinesizeinbytes, width, TIFFout,
 		    &input_compression, &bytesperpixel))
 		return 0;
@@ -376,7 +376,7 @@ cpTiles2Strip(TIFF* in, void * ambiguous_out,
 	if (!inbuf) {
 		TIFFError(TIFFFileName(in),
 				"Error, can't allocate space for image buffer");
-		return (0);
+		return (EXIT_INSUFFICIENT_MEMORY);
 	}
 
 	for (y = ymin ; y < ymin + length + intilelength ; y += intilelength) {
@@ -415,7 +415,7 @@ cpTiles2Strip(TIFF* in, void * ambiguous_out,
 				    "Error, can't read tile at "
 				    UINT32_FORMAT ", " UINT32_FORMAT,
 				    xminoftile, yminoftile);
-				success = 0;
+				error = EXIT_IO_ERROR;
 				goto done;
 			}
 
@@ -438,7 +438,7 @@ cpTiles2Strip(TIFF* in, void * ambiguous_out,
 		if (row_pointers == NULL) {
 			TIFFError(TIFFFileName(in),
 				"Error, can't allocate space for row_pointers");
-			success = 0;
+			error = EXIT_INSUFFICIENT_MEMORY;
 			goto done;
 		}
 
@@ -453,13 +453,13 @@ cpTiles2Strip(TIFF* in, void * ambiguous_out,
 			TIFFStripSize(TIFFout)) < 0) {
 			TIFFError(TIFFFileName(TIFFout),
 				"Error, can't write strip");
-			success = 0;
+			error = EXIT_IO_ERROR;
 		}
 	}
 
 	done:
 	_TIFFfree(inbuf);
-	return success;
+	return error;
 }
 
 
@@ -478,7 +478,7 @@ cpStrips2Strip(TIFF* in, void * ambiguous_out,
 	tsize_t outscanlinesizeinbytes;
 	uint32 y;
 	unsigned char * inbuf, * bufp= outbuf;
-	int success = 1;
+	int error = 0;
 
 	if (output_to_jpeg_rather_than_tiff) {
 		p_cinfo = (struct jpeg_compress_struct *) ambiguous_out;
@@ -488,7 +488,7 @@ cpStrips2Strip(TIFF* in, void * ambiguous_out,
 		TIFFout = (TIFF*) ambiguous_out;
 	}
 
-	if (! testAndFixParameters(in, output_to_jpeg_rather_than_tiff,
+	if (!testAndFixParameters(in, output_to_jpeg_rather_than_tiff,
 		    &outscanlinesizeinbytes, width, TIFFout,
 		    &input_compression, &bytesperpixel))
 		return 0;
@@ -498,7 +498,7 @@ cpStrips2Strip(TIFF* in, void * ambiguous_out,
 	if (!inbuf) {
 		TIFFError(TIFFFileName(in),
 				"Error, can't allocate space for image buffer");
-		return (0);
+		return (EXIT_INSUFFICIENT_MEMORY);
 	}
 
 	/* When compression method doesn't support random access: */
@@ -515,7 +515,7 @@ cpStrips2Strip(TIFF* in, void * ambiguous_out,
 					    "Error, can't read scanline at "
 					    UINT32_FORMAT " for exhausting",
 					    y);
-					success = 0;
+					error = EXIT_IO_ERROR;
 					goto done;
 				} else
 					*y_of_last_read_scanline= y;
@@ -528,7 +528,7 @@ cpStrips2Strip(TIFF* in, void * ambiguous_out,
 				TIFFError(TIFFFileName(in),
 				    "Error, can't read scanline at "
 				    UINT32_FORMAT " for exhausting", y);
-				success = 0;
+				error = EXIT_IO_ERROR;
 				goto done;
 			} else
 				*y_of_last_read_scanline= y;
@@ -543,7 +543,7 @@ cpStrips2Strip(TIFF* in, void * ambiguous_out,
 			TIFFError(TIFFFileName(in),
 			    "Error, can't read scanline at "
 			    UINT32_FORMAT " for copying", y);
-			success = 0;
+			error = EXIT_IO_ERROR;
 			goto done;
 		} else
 			*y_of_last_read_scanline= y;
@@ -564,7 +564,7 @@ cpStrips2Strip(TIFF* in, void * ambiguous_out,
 		if (row_pointers == NULL) {
 			TIFFError(TIFFFileName(in),
 				"Error, can't allocate space for row_pointers");
-			success = 0;
+			error = EXIT_INSUFFICIENT_MEMORY;
 			goto done;
 		}
 
@@ -579,13 +579,13 @@ cpStrips2Strip(TIFF* in, void * ambiguous_out,
 			TIFFStripSize(TIFFout)) < 0) {
 			TIFFError(TIFFFileName(TIFFout),
 				"Error, can't write strip");
-			success = 0;
+			error = EXIT_IO_ERROR;
 		}
 	}
 
 	done:
 	_TIFFfree(inbuf);
-	return success;
+	return error;
 }
 
 
@@ -662,6 +662,7 @@ makeMosaicFromTIFFFile(char * infilename)
 	uint64 outmemorysize, ouroutmemorysize;
 	char * prefix;
 	unsigned char * outbuf = NULL;
+	int return_code = 0;
 
 	in = TIFFOpen(infilename, "r");
 	if (in == NULL) {
@@ -894,6 +895,7 @@ makeMosaicFromTIFFFile(char * infilename)
 			char * outfilename;
 			void * out; /* TIFF* or FILE* */
 			uint32 outlengthwithoverlap;
+			int error = 0;
 
 			uint32 topoverlap = y < voverlap ? y : voverlap;
 			uint32 ywithtopoverlap = y - topoverlap;
@@ -977,21 +979,23 @@ makeMosaicFromTIFFFile(char * infilename)
 				jpeg_start_compress(&cinfo, TRUE);
 
 				if (((TIFFIsTiled(in) &&
-				      cpTiles2Strip(in, &cinfo, 1,
+				      !(error = cpTiles2Strip(in,
+					    &cinfo, 1,
 					    xwithleftoverlap,
 					    ywithtopoverlap,
 					    outwidthwithoverlap,
 					    outlengthwithoverlap,
-					    outbuf)) ||
+					    outbuf))) ||
 				     (!TIFFIsTiled(in) &&
-				      cpStrips2Strip(in, &cinfo, 1,
+				      !(error = cpStrips2Strip(in, 
+					    &cinfo, 1,
 					    xwithleftoverlap,
 					    ywithtopoverlap,
 					    outwidthwithoverlap,
 					    outlengthwithoverlap,
 					    outbuf,
 					    &y_of_last_read_scanline,
-					    inimagelength))))
+					    inimagelength)))))
 					if (verbose)
 						fprintf(stderr,
 							"Piece written.\n");
@@ -1005,19 +1009,19 @@ makeMosaicFromTIFFFile(char * infilename)
 					outwidthwithoverlap);
 				TIFFSetField(out, TIFFTAG_IMAGELENGTH,
 					outlengthwithoverlap);
-				TIFFSetField(out, TIFFTAG_ROWSPERSTRIP, 
+				TIFFSetField(out, TIFFTAG_ROWSPERSTRIP,
 					outlengthwithoverlap);
 
-				if (((TIFFIsTiled(in) &&
-				      cpTiles2Strip(in, out, 0,
-					xwithleftoverlap,
+				if ((TIFFIsTiled(in) &&
+				      !(error = cpTiles2Strip(in, out,
+					0, xwithleftoverlap,
 					ywithtopoverlap,
 					outwidthwithoverlap,
 					outlengthwithoverlap,
-					outbuf)) ||
-				     (! TIFFIsTiled(in) &&
-				      cpStrips2Strip(in, out, 0,
-					xwithleftoverlap,
+					outbuf))) ||
+				     (!TIFFIsTiled(in) &&
+				      !(error = cpStrips2Strip(in, out,
+					0, xwithleftoverlap,
 					ywithtopoverlap,
 					outwidthwithoverlap,
 					outlengthwithoverlap,
@@ -1032,12 +1036,14 @@ makeMosaicFromTIFFFile(char * infilename)
 
 				TIFFClose(out);
 			}
+			if (error && !return_code) /* Error code = 1st error */
+				return_code = error;
 		} /* for y */
 	} /* for x */
 
 	_TIFFfree(outbuf);
 	_TIFFfree(prefix);
-	return 0;
+	return return_code;
 }
 
 
@@ -1155,7 +1161,7 @@ processCompressOptions(char* opt)
 			cp = strchr(cp+1,':');
 		}
 	} else if (strncasecmp(opt, "g3", 2) == 0) {
-		if (! processG3Options(opt))
+		if (!processG3Options(opt))
 			return 0;
 		defcompression = COMPRESSION_CCITTFAX3;
 	} else if (strcasecmp(opt, "g4") == 0) {
@@ -1166,11 +1172,11 @@ processCompressOptions(char* opt)
 			defpredictor = atoi(cp+1);
 		defcompression = COMPRESSION_LZW;
 	} else if (strncasecmp(opt, "zip", 3) == 0) {
-		if (! processZIPOptions(opt))
+		if (!processZIPOptions(opt))
 			return 0;
 		defcompression = COMPRESSION_ADOBE_DEFLATE;
 	}/* else if (strncasecmp(opt, "lzma", 4) == 0) {
-		if (! processZIPOptions(opt) )
+		if (!processZIPOptions(opt) )
 			return 0;
 		defcompression = COMPRESSION_LZMA;
 	}*/ else if (strncasecmp(opt, "jbig", 4) == 0) {
@@ -1388,7 +1394,7 @@ int main(int argc, char * argv[])
 
 	for (; arg < argc ; arg++) {
 		int r= makeMosaicFromTIFFFile(argv[arg]);
-		if (r)
+		if (r && !errorcode) /* Code indicates 1st error */
 			errorcode = r;
 	}
 
